from typing import Annotated
from openai_toolgen import Toolbox

def test_tool_two_args():
    tool = Toolbox()
    @tool("Description how to use foo")
    def foo(
        arg1: Annotated[int, "Description about arg1"],
        arg2: Annotated[str, "Description about arg2"]
    ): pass
    expected_output = [
        {
            "type": "function",
            "function": { 
                "name": "foo",
                "description": "Description how to use foo",
                "parameters": {
                    "type": "object",
                    "properties": {
                        "arg1": {
                            "type": "number",
                            "description": "Description about arg1"
                        },
                        "arg2": {
                            "type": "string",
                            "description": "Description about arg2"
                        }
                    },
                    "required": ["arg1", "arg2"]
                }
            }
        }
    ]
    assert tool.export_all() == expected_output


def test_tool_with_optional_arg():
    tool = Toolbox() 
    @tool("Description how to use foo")
    def foo(
        arg1: Annotated[int, "Description about arg1"],
        arg2: Annotated[str, "Description about arg2"] = "hej"
    ): pass
    assert tool.export_all()[0]['function']['parameters']['required'] == ["arg1"]

def test_tool_with_enum_arg():
    from enum import Enum
    class Unit(str, Enum):
        Celcius = "c"
        Farenheit = "f",
    
    tool = Toolbox()
    @tool
    def get_temperature(unit: Annotated[Unit, "unit desc"]): pass
    assert tool.export_all()[0]['function']['parameters']['properties']['unit'] == {
        'description': 'unit desc',
        'type': 'string', 
        'enum': ["c", "f"]
    }

def test_tool_multiple_funs():
    tool = Toolbox()
    @tool
    def fun1(): pass
    @tool
    def fun2(): pass
    @tool
    def fun3(): pass
    assert len(tool.export_all()) == 3

def test_tool_without_annotation():
    tool = Toolbox()
    @tool
    def get_temperature(unit: str): pass
    assert tool.export_all()[0]['function']['parameters']['properties']['unit'] == {
        'description': '',
        'type': 'string'
    }



from typing import Annotated, Callable, get_origin, get_type_hints, get_args
import inspect
import enum  # Import the enum module

def _map_type(t):
    type_map = {
        int: "number",
        float: "number",
        str: "string",
        bool: "boolean",
        list: "array",
        dict: "object",
        None: "null"
    }
    return type_map.get(t, "string")  # default to string

def _extract_parameter_info(func: Callable) -> dict:
    sig = inspect.signature(func)
    type_hints = get_type_hints(func, globalns=func.__globals__, include_extras=True)
    properties = {}
    required = []
    for param in sig.parameters.values():
        param_name = param.name
        if param.default is inspect.Parameter.empty:
            required.append(param_name)
        annotation = type_hints.get(param_name, None)
        desc = ''
        if annotation:
            if get_origin(annotation) is Annotated:
                base_type, desc = get_args(annotation)
            else:
                base_type = annotation
        else:
            base_type = None
        json_type = _map_type(base_type)
        properties[param_name] = {
            "type": json_type,
            "description": desc
        }
        if isinstance(base_type, enum.EnumMeta):
            properties[param_name]['enum'] = [e.value for e in base_type]
    return {
        "properties": properties,
        "required": required
    }

def _extract_tool_info(func: Callable, desc: str) -> dict:
    parameters = _extract_parameter_info(func)
    return {
        "type": "function",
        "function": {
            "name": func.__name__,
            "description": desc,
            "parameters": {
                "type": "object",
                "properties": parameters['properties'],
                "required": parameters['required']
            }
        }
    }

class Toolbox:
    def __init__(self) -> None:
        self.tools = []

    def __call__(self, *args, **kwargs) -> Callable:
        if len(args) == 1 and callable(args[0]) and not kwargs:
            func=args[0]
            self.tools.append(_extract_tool_info(func, ''))
            return  func
        else:
            desc = args[0] if args else ''
            def decorator(func: Callable):
                self.tools.append(_extract_tool_info(func, desc))
                return func
            return decorator

        self.tools.append(_extract_tool_info(func))
        return func

    def export_all(self) -> list[dict]:
        return self.tools

tool = Toolbox()  # Default
